@@ -3,6 +3,7 @@ use ansi_term::Color::Fixed;
 use regex::Regex;
 use std::cmp::min;
 use std::fmt;
+use std::fs::File;
 use std::num::NonZeroUsize;
 use unicode_segmentation::UnicodeSegmentation;
 
@@ -123,7 +124,6 @@ pub fn string_from_radix(radix: u32) -> String {
 }
 
 
-
 pub fn hex_unless_dec(number:usize, radix:u32) -> String {
     if radix == 10 {
         format!("{}", number)
@@ -134,8 +134,6 @@ pub fn hex_unless_dec(number:usize, radix:u32) -> String {
 }
 
 
-
-
 pub fn hex_unless_dec_with_radix(number:usize, radix:u32) -> String {
     let letter = if radix == 10 {
         'd'
@@ -148,9 +146,6 @@ pub fn hex_unless_dec_with_radix(number:usize, radix:u32) -> String {
 }
 
 
-
-
-
 impl fmt::Debug for State {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "radix: {}|unsaved_changes: {}|show_byte_numbers: {}|show_chars: {}|index: {}|width: {}|n_padding: '{}'|filename: {}|",
@@ -162,6 +157,17 @@ impl fmt::Debug for State {
 
 
 impl State {
+    /// Return the byte numbers necessary for the left column of a display
+    pub fn addresses(&self, from:usize) -> Vec<usize> {
+        // TODO Do this with a generator once
+        // those are supported in Rust
+        let mut to_return:Vec<usize> = vec![];
+        for i in (from..self.all_bytes.len()).step_by(usize::from(self.width)) {
+            to_return.push(i);
+        }
+        to_return
+    }
+
     /// returns index of the byte in the 0-th column of the last row printed
     pub fn print_bytes(&self, range:(usize, usize)) -> Option<usize> {
         if self.empty() {
@@ -184,44 +190,27 @@ impl State {
 
         let bytes = &self.all_bytes[from..=to];
         let max_bytes_line_num = max_bytes_line(bytes, self.width);
-        let mut left_col_byte_num = from;
+        let addresses = self.addresses(from);
         for bytes_line_num in 0..=max_bytes_line_num {
-            if bytes_line_num != 0 {
-                left_col_byte_num += usize::from(self.width);
-            }
             if self.show_byte_numbers {
-                if self.radix == 10 {
-                    print!("{:>5}{}|", left_col_byte_num, self.n_padding);
-                }
-                else {
-                    print!("{:>5x}{}|", left_col_byte_num, self.n_padding);
-                }
+                print!("{}|", address_display(addresses[bytes_line_num], self.radix, &self.n_padding));
             }
-            let cur_line = bytes_line(bytes, bytes_line_num, self.width);
-            let with_color = cur_line.iter().map(|x| formatted_byte(*x, true))
-                    .collect::<Vec<String>>().join(" ");
-            let sans_color = cur_line.iter().map(|x| formatted_byte(*x, false))
-                    .collect::<Vec<String>>().join(" ");
-            if self.color {
-                print!("{}", with_color);
-            }
-            else {
-                print!("{}", sans_color);
-            }
-            let expected_length = usize::from(self.width) * 3 - 1;
-            for _ in num_graphemes(&sans_color)..expected_length {
-                print!(" ");
-            }
-            // TODO Do this padding stuff format!  Unclear why previous attempts
-            // have failed.
+
+            print!(
+                "{}{}",
+                bytes_line(bytes, bytes_line_num, self.width, self.color),
+                bytes_line_padding(bytes, bytes_line_num, self.width)
+            );
+
             if self.show_chars {
                 print!("|   {}", chars_line(bytes, bytes_line_num, self.width,
                     self.color));
             }
+
             println!();
-            left_col_byte_num = from + bytes_line_num * usize::from(self.width);
         }
-        Some(left_col_byte_num)
+
+        Some(addresses[max_bytes_line_num])
     }
 
 
@@ -387,6 +376,44 @@ pub fn max_bytes_line(bytes:&[u8], width:NonZeroUsize) -> usize {
 }
 
 
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_byte_numbers() {
+        let mut state = State {
+            radix: 16,
+            show_byte_numbers: true,
+            show_chars: true,
+            unsaved_changes: true,
+            filename: "filename".to_owned(),
+            show_prompt: true,
+            color: true,
+            index: 0,
+            width: NonZeroUsize::new(0x10).unwrap(),
+            all_bytes: vec![
+                0x00, 0x01, 0x02,
+                0x03, 0x04, 0x05,
+                0x06, 0x07, 0x08,
+                0x09, 0x0a, 0x0b,
+                0x0c, 0x0d, 0x0e,
+                0x0f, 0x10, 0x11,
+                0x12,
+            ],
+            n_padding: "   ".to_owned(),
+        };
+        assert_eq!(state.addresses(0x00),
+            vec![0x00, 0x10]);
+        assert_eq!(state.addresses(0x01),
+            vec![0x01, 0x11]);
+        assert_eq!(state.addresses(0x04),
+            vec![0x04]);
+        state.width = NonZeroUsize::new(3).unwrap();
+        assert_eq!(state.addresses(0x04),
+            vec![0x04, 0x07, 0x0a, 0x0d, 0x10,]);
+    }
+
     #[test]
     fn test_max_bytes_line() {
         let _1 = NonZeroUsize::new(1).unwrap();
@@ -441,9 +468,6 @@ pub fn max_bytes_line(bytes:&[u8], width:NonZeroUsize) -> usize {
     }
 
 
-#[cfg(test)]
-mod tests {
-    use super::*;
 
     #[test]
     fn test_padded_byte() {
@@ -482,55 +506,55 @@ mod tests {
         let _6 = NonZeroUsize::new(6).unwrap();
         let _7 = NonZeroUsize::new(7).unwrap();
         let _8 = NonZeroUsize::new(8).unwrap();
-        assert_eq!(bytes_line(&bytes, 0, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _2).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 1, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 1, _2).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 2, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 2, _2).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 0, _1).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 0, _2).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _1).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _2).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _1).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _2).to_owned(), vec![]);
         let bytes = vec![8, 6, 7, 5, 3, 0, 9,];
-        assert_eq!(bytes_line(&bytes, 0, _1).to_owned(), vec![8,]);
-        assert_eq!(bytes_line(&bytes, 1, _1).to_owned(), vec![6,]);
-        assert_eq!(bytes_line(&bytes, 2, _1).to_owned(), vec![7,]);
-        assert_eq!(bytes_line(&bytes, 3, _1).to_owned(), vec![5,]);
-        assert_eq!(bytes_line(&bytes, 4, _1).to_owned(), vec![3,]);
-        assert_eq!(bytes_line(&bytes, 5, _1).to_owned(), vec![0,]);
-        assert_eq!(bytes_line(&bytes, 6, _1).to_owned(), vec![9,]);
-        assert_eq!(bytes_line(&bytes, 7, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 8, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 9, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _2).to_owned(), vec![8, 6,]);
-        assert_eq!(bytes_line(&bytes, 1, _2).to_owned(), vec![7, 5,]);
-        assert_eq!(bytes_line(&bytes, 2, _2).to_owned(), vec![3, 0,]);
-        assert_eq!(bytes_line(&bytes, 3, _2).to_owned(), vec![9,]);
-        assert_eq!(bytes_line(&bytes, 4, _2).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 5, _2).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _3).to_owned(), vec![8, 6, 7,]);
-        assert_eq!(bytes_line(&bytes, 1, _3).to_owned(), vec![5, 3, 0,]);
-        assert_eq!(bytes_line(&bytes, 2, _3).to_owned(), vec![9,]);
-        assert_eq!(bytes_line(&bytes, 3, _3).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 4, _3).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _4).to_owned(), vec![8, 6, 7, 5,]);
-        assert_eq!(bytes_line(&bytes, 1, _4).to_owned(), vec![3, 0, 9,]);
-        assert_eq!(bytes_line(&bytes, 2, _4).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 3, _4).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 4, _4).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _5).to_owned(), vec![8, 6, 7, 5, 3,]);
-        assert_eq!(bytes_line(&bytes, 1, _5).to_owned(), vec![0, 9,]);
-        assert_eq!(bytes_line(&bytes, 2, _5).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 3, _5).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _6).to_owned(), vec![8, 6, 7, 5, 3, 0,]);
-        assert_eq!(bytes_line(&bytes, 1, _6).to_owned(), vec![9,]);
-        assert_eq!(bytes_line(&bytes, 2, _6).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 3, _6).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _7).to_owned(), vec![8, 6, 7, 5, 3, 0, 9,]);
-        assert_eq!(bytes_line(&bytes, 1, _7).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 2, _7).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 3, _7).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _8).to_owned(), vec![8, 6, 7, 5, 3, 0, 9,]);
-        assert_eq!(bytes_line(&bytes, 1, _8).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 2, _8).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 3, _8).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 0, _1).to_owned(), vec![8,]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _1).to_owned(), vec![6,]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _1).to_owned(), vec![7,]);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _1).to_owned(), vec![5,]);
+        assert_eq!(bytes_line_bytes(&bytes, 4, _1).to_owned(), vec![3,]);
+        assert_eq!(bytes_line_bytes(&bytes, 5, _1).to_owned(), vec![0,]);
+        assert_eq!(bytes_line_bytes(&bytes, 6, _1).to_owned(), vec![9,]);
+        assert_eq!(bytes_line_bytes(&bytes, 7, _1).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 8, _1).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 9, _1).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 0, _2).to_owned(), vec![8, 6,]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _2).to_owned(), vec![7, 5,]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _2).to_owned(), vec![3, 0,]);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _2).to_owned(), vec![9,]);
+        assert_eq!(bytes_line_bytes(&bytes, 4, _2).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 5, _2).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 0, _3).to_owned(), vec![8, 6, 7,]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _3).to_owned(), vec![5, 3, 0,]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _3).to_owned(), vec![9,]);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _3).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 4, _3).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 0, _4).to_owned(), vec![8, 6, 7, 5,]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _4).to_owned(), vec![3, 0, 9,]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _4).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _4).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 4, _4).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 0, _5).to_owned(), vec![8, 6, 7, 5, 3,]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _5).to_owned(), vec![0, 9,]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _5).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _5).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 0, _6).to_owned(), vec![8, 6, 7, 5, 3, 0,]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _6).to_owned(), vec![9,]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _6).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _6).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 0, _7).to_owned(), vec![8, 6, 7, 5, 3, 0, 9,]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _7).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _7).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _7).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 0, _8).to_owned(), vec![8, 6, 7, 5, 3, 0, 9,]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _8).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _8).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _8).to_owned(), vec![]);
     }
 }
 
@@ -547,16 +571,71 @@ pub fn bytes_line_range(bytes:&[u8], line_number:usize, width:NonZeroUsize) -> s
 }
 
 
-pub fn bytes_line(bytes:&[u8], line_number:usize, width:NonZeroUsize) -> &[u8] {
+pub fn bytes_line_bytes(bytes:&[u8], line_number:usize, width:NonZeroUsize) -> &[u8] {
     &bytes[bytes_line_range(bytes, line_number, width)]
 }
 
 
+// TODO Do this padding stuff format!  Unclear why previous attempts
+// have failed.
+pub fn bytes_line_padding(bytes:&[u8], line_num:usize, width:NonZeroUsize) -> String{
+    let mut to_return = String::new();
+    let expected_length = usize::from(width) * 3 - 1;
+    for _ in num_graphemes(&bytes_line(bytes, line_num, width, false))..expected_length {
+        to_return += " ";
+    }
+
+    to_return
+}
+
+pub fn bytes_line(bytes:&[u8], line_number:usize, width:NonZeroUsize, color:bool) -> String {
+    bytes_line_bytes(bytes, line_number, width).iter().map(|x| formatted_byte(*x, color)).collect::<Vec<String>>().join(" ")
+}
+
+
+pub fn chars_line_chars(bytes:&[u8], line_number:usize, width:NonZeroUsize) -> Vec<char> {
+    let mut to_return:Vec<char> = vec![];
+    for index in bytes_line_range(bytes, line_number, width) {
+        to_return.push(chared_byte(bytes[index]));
+    }
+
+    to_return
+}
+
+
+pub fn colored_chared_bytes(bytes:&[u8], line_number:usize, width:NonZeroUsize) -> Vec<String> {
+    let mut to_return:Vec<String> = vec![];
+    for index in bytes_line_range(bytes, line_number, width) {
+            to_return.push(String::from(colored_chared_byte(bytes[index])));
+    }
+
+    to_return
+}
+
+
+pub fn chared_bytes(bytes:&[u8], line_number:usize, width:NonZeroUsize) -> Vec<char> {
+    let mut to_return:Vec<char> = vec![];
+    for index in bytes_line_range(bytes, line_number, width) {
+        to_return.push(chared_byte(bytes[index]));
+    }
+
+    to_return
+}
+
+
 pub fn chars_line(bytes:&[u8], line_number:usize, width:NonZeroUsize, color:bool) -> String {
     let mut to_return:String = "".to_owned();
-    for index in bytes_line_range(bytes, line_number, width) {
-        to_return += &String::from(chared_byte(bytes[index], color));
+    if color {
+        for colored_char in colored_chared_bytes(bytes, line_number, width) {
+            to_return += &colored_char;
+        }
     }
+    else {
+        for chared in chared_bytes(bytes, line_number, width) {
+            to_return += &chared.to_string();
+        }
+    }
+
     to_return
 }
 
@@ -571,13 +650,13 @@ fn formatted_byte(byte:u8, color:bool) -> String {
 }
 
 
-fn chared_byte(byte:u8, color:bool) -> String {
-    if color {
-        Byte(byte).color().paint(String::from(Byte(byte).as_char())).to_string()
-    }
-    else {
-        Byte(byte).as_char().to_string()
-    }
+fn colored_chared_byte(byte:u8) -> String {
+    Byte(byte).color().paint(String::from(Byte(byte).as_char())).to_string()
+}
+
+
+fn chared_byte(byte:u8) -> char {
+    Byte(byte).as_char()
 }
 
 
@@ -589,3 +668,40 @@ pub fn num_graphemes(unicode_string: &str) -> usize {
     return unicode_string.graphemes(true).count();
 }
 
+
+/// Print version if compiled from Cargo
+pub fn print_cargo_version() {
+    if let Some(version) = option_env!("CARGO_PKG_VERSION") {
+        println!("{}", version);
+    }
+    else {
+        println!("Version unknown (not compiled with cargo)");
+    }
+}
+
+
+pub fn address_display(address: usize, radix:u32, padding:&str) -> String {
+    if radix == 10 {
+        format!("{:>5}{}", address, padding)
+    }
+    else {
+        format!("{:>5x}{}", address, padding)
+    }
+}
+
+
+pub fn filehandle(filename:&str) -> Result<Option<File>, String> {
+    match File::open(filename) {
+        Ok(filehandle) => {
+            Ok(Some(filehandle))
+        },
+        Err(error) => {
+            if error.kind() == std::io::ErrorKind::NotFound {
+                Ok(None)
+            }
+            else {
+                Err(format!("Error opening '{}'", filename))
+            }
+        },
+    }
+}
